# Generated by Django 4.2.11 on 2024-04-25 15:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0093_uploadedfile'),
        ('tests', '0036_complexdefaultstreampage'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPermissionedGenericSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('sensitive_email', models.EmailField(max_length=50)),
            ],
            options={
                'permissions': [('can_edit_sensitive_email_generic_setting', 'Can edit sensitive email generic setting.')],
            },
        ),
        migrations.AlterModelOptions(
            name='featurecompletetoy',
            options={'permissions': [('can_set_release_date', 'Can set release date')]},
        ),
        migrations.CreateModel(
            name='TestPermissionedSiteSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('sensitive_email', models.EmailField(max_length=50)),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'permissions': [('can_edit_sensitive_email_site_setting', 'Can edit sensitive email site setting.')],
            },
        ),
    ]
