{% load wagtailadmin_tags %}
<label class="switch">
    {% include "django/forms/widgets/input.html" %}
    <span class="switch__toggle">
        {% icon name="check" classname="switch__icon" %}
    </span>
</label>
