# Generated by Django 4.2.5 on 2023-09-26 09:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("tests", "0028_fullfeaturedsnippet_some_number"),
    ]

    operations = [
        migrations.AddField(
            model_name="variousondeletemodel",
            name="cascading_toy",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="tests.featurecompletetoy",
            ),
        ),
    ]
