# Generated by Django 4.2.6 on 2023-10-27 18:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailsearch", "0007_delete_editorspick"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="querydailyhits",
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name="querydailyhits",
            name="query",
        ),
        migrations.DeleteModel(
            name="Query",
        ),
        migrations.DeleteModel(
            name="QueryDailyHits",
        ),
    ]
