# Generated by Django 5.0.7 on 2024-08-06 07:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0042_alter_customdocument_file_size_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="customimage",
            name="description",
            field=models.CharField(
                blank=True, default="", max_length=255, verbose_name="description"
            ),
        ),
        migrations.AddField(
            model_name="customimagefilepath",
            name="description",
            field=models.CharField(
                blank=True, default="", max_length=255, verbose_name="description"
            ),
        ),
        migrations.AddField(
            model_name="customimagewithauthor",
            name="description",
            field=models.CharField(
                blank=True, default="", max_length=255, verbose_name="description"
            ),
        ),
        migrations.AddField(
            model_name="customrestaurantimage",
            name="description",
            field=models.CharField(
                blank=True, default="", max_length=255, verbose_name="description"
            ),
        ),
    ]
