(()=>{"use strict";var e,t={6788:(e,t,r)=>{var l=r(1669),a=r.n(l);a()((()=>{a()('[data-widget="filtered-select"]').each((function(){const e=a()("#"+this.dataset.filterField),t=a()(this),r=[];function l(){const l=t.val();t.empty();const o=e.val();let i;if(""===o)i=r;else{i=[];for(let e=0;e<r.length;e+=1)""!==r[e].value&&-1===r[e].filterValue.indexOf(o)||i.push(r[e])}let n=!1;for(let e=0;e<i.length;e+=1){const r=a()("<option>");r.attr("value",i[e].value),i[e].value===l&&(n=!0),r.text(i[e].label),t.append(r)}n?t.val(l):t.val("")}a()("option",this).each((function(){let e;e="filterValue"in this.dataset?this.dataset.filterValue.split(","):[],r.push({value:this.value,label:this.label,filterValue:e})})),l(),e.change(l)}))}))},1669:e=>{e.exports=jQuery}},r={};function l(e){var a=r[e];if(void 0!==a)return a.exports;var o=r[e]={exports:{}};return t[e](o,o.exports,l),o.exports}l.m=t,e=[],l.O=(t,r,a,o)=>{if(!r){var i=1/0;for(f=0;f<e.length;f++){for(var[r,a,o]=e[f],n=!0,u=0;u<r.length;u++)(!1&o||i>=o)&&Object.keys(l.O).every((e=>l.O[e](r[u])))?r.splice(u--,1):(n=!1,o<i&&(i=o));if(n){e.splice(f--,1);var s=a();void 0!==s&&(t=s)}}return t}o=o||0;for(var f=e.length;f>0&&e[f-1][2]>o;f--)e[f]=e[f-1];e[f]=[r,a,o]},l.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return l.d(t,{a:t}),t},l.d=(e,t)=>{for(var r in t)l.o(t,r)&&!l.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},l.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),l.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),l.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},l.j=491,(()=>{var e={491:0};l.O.j=t=>0===e[t];var t=(t,r)=>{var a,o,[i,n,u]=r,s=0;if(i.some((t=>0!==e[t]))){for(a in n)l.o(n,a)&&(l.m[a]=n[a]);if(u)var f=u(l)}for(t&&t(r);s<i.length;s++)o=i[s],l.o(e,o)&&e[o]&&e[o][0](),e[o]=0;return l.O(f)},r=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})();var a=l.O(void 0,[321],(()=>l(6788)));a=l.O(a)})();