# Generated by Django 4.0.4 on 2022-09-09 14:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0008_modelwithstringtypeprimarykey"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="eventpage",
            options={
                "permissions": [
                    ("custom_see_panel_setting", "Can see the panel."),
                    ("other_custom_see_panel_setting", "Can see the panel."),
                ]
            },
        ),
    ]
