{% load i18n wagtailadmin_tags %}
<td>
    {% formattedfield field=form.page sr_only_label=True %}
    {{ form.DELETE }}
</td>

<td>
    <button class="button button-small no" type="button" id="{{ form.DELETE.id_for_label }}-button" data-action="w-formset#delete">{% trans "Delete" %}</button>
</td>
