# Generated by Django 5.1.4 on 2025-01-19 21:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0045_alter_streampage_body"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="customrendition",
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name="customrendition",
            constraint=models.UniqueConstraint(
                fields=("image", "filter_spec", "focal_point_key"),
                name="unique_rendition",
            ),
        ),
    ]
