# Generated by Django 5.0.6 on 2024-09-23 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailusers", "0013_userprofile_density"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="contrast",
            field=models.CharField(
                choices=[("system", "System default"), ("more_contrast", "More contrast")],
                default="system",
                max_length=40,
                verbose_name="contrast",
            ),
        ),
    ]
