{% load wagtailadmin_tags i18n %}

<a href="{{ history_url }}"
   class="{{ nav_icon_button_classes }}"
   aria-label="{% trans 'History' %}"
   data-controller="w-tooltip"
   data-w-tooltip-content-value="{% trans 'History' %}"
   data-w-tooltip-offset-value="[0, 0]"
>
    {% icon name="history" classname=nav_icon_classes %}
</a>
