# Generated by Django 4.0.3 on 2022-09-02 03:18

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        # changed dependency to last wagtail3 migration
        ("wagtailcore", "0069_log_entry_jsonfield"),
    ]

    operations = [
        migrations.CreateModel(
            name="SampleModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "content",
                    wagtail.fields.StreamField(
                        [
                            ("char1", wagtail.blocks.CharBlock()),
                            ("char2", wagtail.blocks.CharBlock()),
                            (
                                "simplestruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplestream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplelist",
                                wagtail.blocks.ListBlock(wagtail.blocks.CharBlock()),
                            ),
                            (
                                "nestedstruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedstream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedlist_struct",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StructBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                            (
                                "nestedlist_stream",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StreamBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                        ],
                        use_json_field=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SamplePage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "content",
                    wagtail.fields.StreamField(
                        [
                            ("char1", wagtail.blocks.CharBlock()),
                            ("char2", wagtail.blocks.CharBlock()),
                            (
                                "simplestruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplestream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        ("char2", wagtail.blocks.CharBlock()),
                                    ]
                                ),
                            ),
                            (
                                "simplelist",
                                wagtail.blocks.ListBlock(wagtail.blocks.CharBlock()),
                            ),
                            (
                                "nestedstruct",
                                wagtail.blocks.StructBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedstream",
                                wagtail.blocks.StreamBlock(
                                    [
                                        ("char1", wagtail.blocks.CharBlock()),
                                        (
                                            "stream1",
                                            wagtail.blocks.StreamBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "struct1",
                                            wagtail.blocks.StructBlock(
                                                [
                                                    (
                                                        "char1",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                    (
                                                        "char2",
                                                        wagtail.blocks.CharBlock(),
                                                    ),
                                                ]
                                            ),
                                        ),
                                        (
                                            "list1",
                                            wagtail.blocks.ListBlock(
                                                wagtail.blocks.CharBlock()
                                            ),
                                        ),
                                    ]
                                ),
                            ),
                            (
                                "nestedlist_struct",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StructBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                            (
                                "nestedlist_stream",
                                wagtail.blocks.ListBlock(
                                    wagtail.blocks.StreamBlock(
                                        [
                                            ("char1", wagtail.blocks.CharBlock()),
                                            ("char2", wagtail.blocks.CharBlock()),
                                        ]
                                    )
                                ),
                            ),
                        ],
                        use_json_field=True,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]
