# Generated by Django 5.0.9 on 2024-10-09 16:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildocs", "0013_delete_uploadeddocument"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="file_size",
            field=models.PositiveBigIntegerField(editable=False, null=True),
        ),
    ]
