(()=>{"use strict";var e,t={4967:(e,t,r)=>{var o=r(1669),i=r.n(o);i()((()=>{i()('[data-a11y-dialog-show="set-privacy"]').on("click",(function(){return ModalWorkflow({dialogId:"set-privacy",url:this.getAttribute("data-url"),onload:{set_privacy(e){i()("form",e.body).on("submit",(function(){return e.postForm(this.action,i()(this).serialize()),!1}));const t=i()("input[name='restriction_type'][value='password']",e.body),r=i()("input[name='restriction_type'][value='groups']",e.body),o=i()('[name="password"]',e.body).parents("[data-field-wrapper]"),n=i()("#groups-fields",e.body);function a(){t.is(":checked")?(o.show(),n.hide()):r.is(":checked")?(o.hide(),n.show()):(o.hide(),n.hide())}a(),i()("input[name='restriction_type']",e.body).on("change",a)},set_privacy_done(e,{is_public:t}){document.dispatchEvent(new CustomEvent("w-privacy:changed",{bubbles:!0,cancelable:!1,detail:{isPublic:t}})),e.close()}}}),!1}))}))},1669:e=>{e.exports=jQuery}},r={};function o(e){var i=r[e];if(void 0!==i)return i.exports;var n=r[e]={exports:{}};return t[e](n,n.exports,o),n.exports}o.m=t,e=[],o.O=(t,r,i,n)=>{if(!r){var a=1/0;for(d=0;d<e.length;d++){for(var[r,i,n]=e[d],s=!0,l=0;l<r.length;l++)(!1&n||a>=n)&&Object.keys(o.O).every((e=>o.O[e](r[l])))?r.splice(l--,1):(s=!1,n<a&&(a=n));if(s){e.splice(d--,1);var u=i();void 0!==u&&(t=u)}}return t}n=n||0;for(var d=e.length;d>0&&e[d-1][2]>n;d--)e[d]=e[d-1];e[d]=[r,i,n]},o.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return o.d(t,{a:t}),t},o.d=(e,t)=>{for(var r in t)o.o(t,r)&&!o.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},o.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),o.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),o.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},o.j=658,(()=>{var e={658:0};o.O.j=t=>0===e[t];var t=(t,r)=>{var i,n,[a,s,l]=r,u=0;if(a.some((t=>0!==e[t]))){for(i in s)o.o(s,i)&&(o.m[i]=s[i]);if(l)var d=l(o)}for(t&&t(r);u<a.length;u++)n=a[u],o.o(e,n)&&e[n]&&e[n][0](),e[n]=0;return o.O(d)},r=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})();var i=o.O(void 0,[321],(()=>o(4967)));i=o.O(i)})();