.typed-table-block .w-field__wrapper{margin-bottom:0}.typed-table-block table{margin:2.5rem 0 1rem 2.5rem}.typed-table-block td,.typed-table-block th{padding:.25rem}.typed-table-block th{border:2px solid var(--w-color-border-field-default);position:relative}.typed-table-block th button.append-column,.typed-table-block th button.prepend-column{inset-inline-start:-.9rem;position:absolute;top:-2rem}.typed-table-block th button.delete-column{inset-inline-end:.625rem;position:absolute;top:.875rem}.typed-table-block th input.column-heading{font-weight:700;padding-inline-end:3rem}.typed-table-block td{border-inline-start:1px solid var(--w-color-border-field-default);position:relative}.typed-table-block td:first-child,.typed-table-block th:first-child,.typed-table-block th:last-child{border-width:0}.typed-table-block tbody tr,.typed-table-block tfoot tr{border-top:1px dotted var(--w-color-border-field-default);position:relative}.typed-table-block tbody tr button.prepend-row,.typed-table-block tfoot tr button.prepend-row{inset-inline-start:-2rem;position:absolute;top:-.9rem}.typed-table-block tbody tr button.delete-row,.typed-table-block tfoot tr button.delete-row{margin-inline-start:.25rem}.typed-table-block tbody tr td:not(.control-cell),.typed-table-block tfoot tr td:not(.control-cell){min-width:20rem;vertical-align:top}.typed-table-block ul.add-column-menu{background-color:var(--w-color-surface-page);inset-inline-start:-.9rem;min-width:10rem;position:absolute;top:0;z-index:100}.typed-table-block ul.add-column-menu button{margin:1px;width:100%}.typed-table-block .control-cell ul.add-column-menu{top:0}.typed-table-block .w-field--commentable{padding-inline-end:2rem}.typed-table-block .w-field--draftail_rich_text_area{padding-inline-start:2.25rem}.typed-table-block__wrapper{min-height:20rem;overflow-x:auto}
