{% load l10n wagtailadmin_tags %}
{% load wagtailcore_tags %}
<td class="{% if column.classname %}{{ column.classname }} {% endif %}title">
    <div class="title-wrapper">
        {% if page.can_choose %}
            {% if column.is_multiple_choice %}
                <label for="chooser-modal-select-{{ page.id|unlocalize }}">{{ value }}</label>
            {% else %}
                <a class="choose-page" href="#{{ page.id|unlocalize }}" data-id="{{ page.id|unlocalize }}" data-title="{{ page.title }}" data-admin-title="{{ page.get_admin_display_title }}" data-url="{% pageurl page %}" data-parent-id="{{ page.get_parent.id|unlocalize }}" data-edit-url="{% url 'wagtailadmin_pages:edit' page.id %}">{{ value }}</a>
            {% endif %}
        {% else %}
            {{ value }}
        {% endif %}
        {% if show_locale_labels %}
            {% status page.locale.get_display_name classname="w-status--label" %}
        {% endif %}

        {% include "wagtailadmin/pages/listing/_privacy_indicator.html" with page=page %}
        {% include "wagtailadmin/pages/listing/_locked_indicator.html" with page=page %}
    </div>
</td>
