{% if value is None %}
    {{ value }}
{% else %}
    <a href="{{ value.url }}">
        {{ value.title }}
        <span class="meta">
            ({{ value.file_extension|upper }}, {{ value.file.size|filesizeformat }})
        </span>
    </a>
{% endif %}
