{% load i18n wagtailadmin_tags %}
{% trans "Total submissions" as total %}
{% trans "Latest submission" as latest %}
<p>
    <span class="w-label-3">{{ total }}</span> <a href="{% url 'wagtailforms:list_submissions' self.instance.id %}">{{ submission_count|intcomma }}</a>
</p>
<p>
    <span class="w-label-3">{{ latest }}</span> {{ last_submit_time }}
</p>
