# Generated by Django 4.0.10 on 2023-11-10 01:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("searchtests", "0002_bookunindexed"),
    ]

    operations = [
        migrations.AddField(
            model_name="book",
            name="summary",
            field=models.TextField(blank=True),
        ),
    ]
