# Generated by Django 2.1.13 on 2019-10-17 13:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0043_lock_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="grouppagepermission",
            name="permission_type",
            field=models.CharField(
                choices=[
                    ("add", "Add/edit pages you own"),
                    ("edit", "Edit any page"),
                    ("publish", "Publish any page"),
                    ("bulk_delete", "Delete pages with children"),
                    ("lock", "Lock/unlock pages you've locked"),
                    ("unlock", "Unlock any page"),
                ],
                max_length=20,
                verbose_name="permission type",
            ),
        ),
    ]
