{% templatetag openblock %} extends "base.html" {% templatetag closeblock %}

{% templatetag openblock %} block title {% templatetag closeblock %}Page not found{% templatetag openblock %} endblock {% templatetag closeblock %}

{% templatetag openblock %} block body_class {% templatetag closeblock %}template-404{% templatetag openblock %} endblock {% templatetag closeblock %}

{% templatetag openblock %} block content {% templatetag closeblock %}
<h1>Page not found</h1>

<h2>Sorry, this page could not be found.</h2>
{% templatetag openblock %} endblock {% templatetag closeblock %}
