{% extends "wagtailadmin/generic/index_results.html" %}
{% load i18n wagtailadmin_tags %}

{% block no_results_message %}
    {% if is_searching %}
        <h2 role="alert">{% blocktrans trimmed %}Sorry, no documents match "<em>{{ query_string }}</em>"{% endblocktrans %}</h2>
    {% else %}
        {% with wagtaildocs_add_document_url=add_url %}
            {% if current_collection %}
                <p>{% blocktrans trimmed %}You haven't uploaded any documents in this collection. Why not <a href="{{ wagtaildocs_add_document_url }}">upload one now</a>?{% endblocktrans %}</p>
            {% else %}
                <p>{% blocktrans trimmed %}You haven't uploaded any documents. Why not <a href="{{ wagtaildocs_add_document_url }}">upload one now</a>?{% endblocktrans %}</p>
            {% endif %}
        {% endwith %}
    {% endif %}
{% endblock %}
