# Generated by Django 5.1.5 on 2025-02-13 01:36

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0046_alter_custom_rendition_to_unique_constraint"),
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.AddField(
            model_name="advertwithcustomuuidprimarykey",
            name="page",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="wagtailcore.page",
            ),
        ),
    ]
