{% load wagtailadmin_tags %}

{% if view.active_filters %}
    {% include "wagtailadmin/shared/active_filters.html" with active_filters=view.active_filters %}
{% endif %}

{% if render_filters_fragment %}
    <template data-controller="w-teleport" data-w-teleport-target-value="#filters-drilldown" data-w-teleport-reset-value="true">
        {% include "wagtailadmin/shared/headers/_filters.html" with filters=filters %}
    </template>
{% endif %}

<form id="page-reorder-form">
    {% csrf_token %}

    {% block before_results %}{% endblock %}

    {% component table %}

    {% if is_paginated %}
        {% include "wagtailadmin/shared/pagination_nav.html" with items=page_obj linkurl=index_url %}
    {% endif %}
</form>
