<div class="w-field__textoutput" role="textbox" tabindex="0" aria-labelledby="{{ id_for_label }}-label"{% if help_text %} aria-describedby="{{ help_text_id }}"{% endif %} aria-readonly="true">
    {{ display_value }}
</div>
