{% load i18n wagtailadmin_tags %}

<p>{% trans "Choose which type of task you'd like to create." %}</p>

{% if task_types %}
    <ul class="listing">
        {% for verbose_name, app_label, model_name, description in task_types %}
            <li>
                <div class="row row-flush">
                    <div class="col6">
                        <a href="{% url 'wagtailadmin_workflows:task_chooser_create' %}?create_model={{ app_label|urlencode }}.{{ model_name|urlencode }}" class="task-type-choice">{% icon name="plus-inverse" classname="middle w-mr-1" %}{{ verbose_name }}</a>
                    </div>
                    <div class="col6">
                        {{ description }}
                    </div>
                </div>
            </li>
        {% endfor %}
    </ul>
{% endif %}
