{% load wagtailadmin_tags %}

{% if show_menu %}
    {% dropdown_button button=default_menu_item toggle_icon="arrow-up" keep_mounted=True %}
        {% if menu_items_fragment %}
            {{ menu_items_fragment }}
        {% endif %}
        {% for item in rendered_menu_items %}
            {{ item }}
        {% endfor %}
    {% enddropdown_button %}
{% elif default_menu_item %}
    {{ default_menu_item }}
{% endif %}
