# Generated by Django 3.1.3 on 2020-11-27 01:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0059_apply_collection_ordering"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="workflowstate",
            name="unique_in_progress_workflow",
        ),
        migrations.AddConstraint(
            model_name="workflowstate",
            constraint=models.UniqueConstraint(
                condition=models.Q(status__in=("in_progress", "needs_changes")),
                fields=("page",),
                name="unique_in_progress_workflow",
            ),
        ),
    ]
