<!DOCTYPE html>
<html lang="en">
    <head>
        <title>{{ object.text }} (Default Preview)</title>
    </head>
    <body>
        <h1 id="current-object">
            {{ object.text }}
        </h1>
        <span id="previewablesitesetting">
            {{ settings.tests.PreviewableSiteSetting.text }}
        </span>
        <span id="previewablegenericsetting">
            {{ settings.tests.PreviewableGenericSetting.text }}
        </span>
    </body>
</html>
