# Generated by Django 4.2.6 on 2023-10-26 15:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0090_remove_grouppagepermission_permission_type"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="revision",
            name="submitted_for_moderation",
        ),
    ]
