# Generated by Django 5.1.3 on 2025-04-21 10:43

from django.db import migrations, models

import wagtail.models.preview


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0054_delete_notyetregisteredgenericsetting_and_more"),
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.CreateModel(
            name="PreviewableGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.models.preview.PreviewableMixin, models.Model),
        ),
        migrations.CreateModel(
            name="PreviewableSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.models.preview.PreviewableMixin, models.Model),
        ),
    ]
