{% load wagtailadmin_tags %}

<td class="{% if column.classname %}{{ column.classname }} {% endif %}title">
    <div class="title-wrapper">
        {% fragment as title %}
            {% if title_id %}<span id="{{ title_id }}">{{ value }}</span>
            {% else %}{{ value }}
            {% endif %}
        {% endfragment %}
        {% block title %}
            {% if link_url %}
                <a {% include "wagtailadmin/shared/attrs.html" with attrs=link_attrs %}>{{ title }}</a>
            {% elif label_id %}
                <label for="{{ label_id }}">{{ title }}</label>
            {% else %}
                {{ title }}
            {% endif %}
        {% endblock %}
    </div>
    {% block extra %}
        {% if buttons %}
            <ul class="actions">
                {% block buttons %}
                    {% include "wagtailadmin/shared/buttons.html" %}
                {% endblock %}
            </ul>
        {% endif %}
    {% endblock %}
</td>
