# Generated by Django 5.2.3 on 2025-07-23 16:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailusers", "0014_userprofile_contrast"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="keyboard_shortcuts",
            field=models.BooleanField(
                default=True,
                help_text="Enable custom keyboard shortcuts specific to Wagtail.",
                verbose_name="Keyboard shortcuts",
            ),
        ),
    ]
