# Generated by Django 4.0.10 on 2023-09-07 12:46

from django.db import migrations, models
import wagtail.search.index


class Migration(migrations.Migration):

    dependencies = [
        ("snippetstests", "0010_delete_filterablesnippet"),
    ]

    operations = [
        migrations.CreateModel(
            name="NonAutocompleteSearchableSnippet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=255)),
            ],
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
    ]
