# Generated by Django 4.2.dev20221212120043 on 2022-12-12 13:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0078_referenceindex"),
    ]

    operations = [
        migrations.RenameField(
            model_name="taskstate",
            old_name="page_revision",
            new_name="revision",
        ),
    ]
