# Generated by Django 5.0.1 on 2024-01-30 18:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtaildocs', '0012_uploadeddocument'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UploadedDocument',
        ),
    ]
