# Generated by Django 5.1.10 on 2025-06-10 20:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0053_pagewithexcludedcopyfield_special_stream_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="NotYetRegisteredGenericSetting",
        ),
        migrations.DeleteModel(
            name="NotYetRegisteredSiteSetting",
        ),
    ]
