# Generated by Django 4.0.3 on 2022-05-06 04:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0075_populate_latest_revision_and_revision_object_str"),
    ]

    operations = [
        migrations.AddField(
            model_name="modellogentry",
            name="revision",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="wagtailcore.revision",
            ),
        ),
    ]
